#include <iostream>
#include <vector>
#include <string>

using namespace std;

class User {
public:
	friend class Database;
	User(const User&) = delete;
	User& operator=(const User&) = delete;

	int getId() const { return id_; }
	string getUsername() const { return username_; }
	string getPassword() const { return password_; }
	string changePassword(const string& password) {
		password_ = password;
	}

private:
	User(const string& username, const string& password): username_(username), password_(password){}

	static int ID;
	int id_ = ++ID;
	string username_, password_;
};

int User::ID = 0;

class PrintStrategy {
public:
	virtual void print(vector<User*>&) const = 0;
};

class ClassicPrintStrategy: public PrintStrategy {
	void print(vector<User*>& users) const {
		cout << "Users: " << endl;
		for (User* user : users) {
			cout << user->getId() << ": " << user->getUsername() << " " << user->getPassword() << endl;
		}
	}
};

class EncryptedPrintStrategy: public PrintStrategy {
	void print(vector<User*>& users) const {
		cout << "Users: " << endl;
		for (User* user : users) {
			cout << user->getId() << ": " << user->getUsername() << " " << "*************" << endl;
		}
	}
};

// Baza - jedna jedina u sistemu => singleton
class Database {
public:
	Database(const Database&) = delete;
	Database& operator=(const Database&) = delete;
	~Database() {
		for (User* user : users_) delete user;
		if (strategy_ != nullptr) delete strategy_;
		cout << "Database destroyed!" << endl;
	}

	static Database& getInstance() {
		static Database instance;
		return instance;
	}

	User* createUser(const string& username, const string& password) {
		User* user = new User(username, password);
		users_.push_back(user);
		return user;
	}

	void printDatabaseContent() {
		strategy_->print(users_);
	}

	void setPrintStrategy(PrintStrategy* strategy) {
		if (strategy_ != nullptr) delete strategy_;
		strategy_ = strategy;
	}

private:
	Database() { cout << "Database created!" << endl << endl; };
	vector<User*> users_;
	PrintStrategy* strategy_ = nullptr;
};

int main() {

	Database::getInstance().createUser("pera", "pera123");
	Database::getInstance().createUser("zika", "zika123");
	Database::getInstance().createUser("mika", "mika123");

	Database::getInstance().setPrintStrategy(new ClassicPrintStrategy());

	Database::getInstance().printDatabaseContent();

	Database::getInstance().setPrintStrategy(new EncryptedPrintStrategy());

	Database::getInstance().printDatabaseContent();

	return 0;
}